/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/

/**
 * Creates the drawwidget and sets the drawing area coordinate validators
 */
void curvedlg::init()
{
    drawWidget = new drawwidget(graphframe,"");

    editlX->setValidator(new QDoubleValidator(-1e100,1e100,2,editlX) );
    editlY->setValidator(new QDoubleValidator(-1e100,1e100,2,editlY) );
    editrX->setValidator(new QDoubleValidator(-1e100,1e100,2,editrX) );
    editrY->setValidator(new QDoubleValidator(-1e100,1e100,2,editrY) );

}


/**
 * Initializes the drawing data and set the drawable area coordinates
 * @param pmatData pointer to the mathematical class
 */
void curvedlg::initmatdata( matdata * pmatData)
{
    matData = pmatData;
    drawWidget->initmatdata(matData);

    autoset_coords();
}

/**
 * This method is called when the drawable coordinates changed.
 * Sets the internal variables and initiates a redraw.
 */
void curvedlg::set_coords()
{
    char cbuff[100];
    sprintf(cbuff,"%8.5g",c_x1);    editlX->setText( QString(cbuff) );
    sprintf(cbuff,"%8.5g",c_y1);    editlY->setText( QString(cbuff) );
    sprintf(cbuff,"%8.5g",c_x2);    editrX->setText( QString(cbuff) );
    sprintf(cbuff,"%8.5g",c_y2);    editrY->setText( QString(cbuff) );

    drawWidget->set_coords(c_x1,c_y1,c_x2,c_y2);
}


/**
 * Calculates the drawable area size to be able to display all of the data points.
 * Initiates a redraw.
 */
void curvedlg::autoset_coords()
{
    int i;

    c_x1 = matData->points[0].x;
    c_x2 = matData->points[0].x;
    c_y1 = matData->points[0].y;
    c_y2 = matData->points[0].y;

    for(i=0;i<matData->num_points;i++) {
    c_x1 = c_x1>matData->points[i].x? matData->points[i].x  : c_x1;
    c_y1 = c_y1>matData->points[i].y? matData->points[i].y  : c_y1;
    c_x2 = c_x2<matData->points[i].x? matData->points[i].x  : c_x2;
    c_y2 = c_y2<matData->points[i].y? matData->points[i].y  : c_y2;
    }

    if (c_x1 == c_x2)  { c_x1--; c_x2++; }

    if (c_y1 == c_y2) { c_y1--; c_y2++; }

    set_coords();
}


/**
 * Slot is called when the focus leaves the input.
 * Sets the coordinate and initiate a redraw.
 */
void curvedlg::editlX_lostFocus()
{
    c_x1 = editlX->text().toDouble();
    drawWidget->set_coords(c_x1,c_y1,c_x2,c_y2);
}


/**
 * Slot is called when the focus leaves the input.
 * Sets the coordinate and initiate a redraw.
 */
void curvedlg::editlY_lostFocus()
{
    c_y1 = editlY->text().toDouble();
    drawWidget->set_coords(c_x1,c_y1,c_x2,c_y2);
}


/**
 * Slot is called when the focus leaves the input.
 * Sets the coordinate and initiate a redraw.
 */
void curvedlg::editrX_lostFocus()
{
    c_x2= editrX->text().toDouble();
    drawWidget->set_coords(c_x1,c_y1,c_x2,c_y2);
}


/**
 * Slot is called when the focus leaves the input.
 * Sets the coordinate and initiate a redraw.
 */
void curvedlg::editrY_lostFocus()
{
    c_y2 = editrY->text().toDouble();
    drawWidget->set_coords(c_x1,c_y1,c_x2,c_y2);
}

/**
 * Event dispatcher that filters QMOUSECOORDEVENT.
 * Displays the mouse coordinates and the function value in the dialog.
 * @param e Event
 */
bool curvedlg::event( QEvent * e )
{
    QPoint mp;
    QString s;
    long double y,ty,f_x;
    int i;

    if ( e->type() == QMOUSECOORDEVENT ) {
            qmousecoordsevent* mce = (qmousecoordsevent *)e;
            mp = mce->pos();

            s.sprintf("%8.5Lg",drawWidget->X_pos_to_coord(mp.x()) );
            mouseX->setText( s );

            s.sprintf("%8.5Lg",drawWidget->Y_pos_to_coord(mp.y()) );
            mouseY->setText( s );


            y = 0;
            for(i=0;i<=matData->num_ad;i++){
                f_x = drawWidget->X_pos_to_coord(mp.x());
                ty = matData->coefs[i] * powl(f_x,i);
                y += ty;
            }

            s.sprintf("%13.4Lg", y );
            lnFx->setText( s );

            return TRUE;
    } else
            return QDialog::event(e);
}



/**
 * Slot is called when mathematical data changed.
 * Redisplays calculated values and redraws the function curve.
 */
void curvedlg::matdata_changed()
{
    int i;
    char chbuff[100];

    lbcoefs->clear();
    for(i=0; i <=matData->num_ad;i++) {
        sprintf(chbuff,"x^%i = %13.4Lg",i,matData->coefs[i]);
        lbcoefs->insertItem( QString( chbuff ));
    }

    lbcoords->clear();
    for(i=0; i <matData->num_points;i++) {
        sprintf(chbuff,"( %13.4Lg ; %13.4Lg )",matData->points[i].x,matData->points[i].y);
//        sprintf(chbuff,"(%13.4Lg,%i,%i)",matData->points[i].x,matData->num_points,i);
        lbcoords->insertItem( QString( chbuff));
    }


    sprintf(chbuff,"%13.4Lg",matData->sigma);
    le_sigma->setText( QString( chbuff ) );

    sprintf(chbuff,"%13.4Lg",matData->sigma_star);
    le_sigma_star->setText( QString( chbuff ) );

    drawWidget->update();
}

/**
 * Scrolls up the drawable area.
 */
void curvedlg::btnScrollUp_clicked()
{
    long double dy;

    dy = (c_y2 - c_y1);

    c_y2 += 0.05 * dy;
    c_y1 += 0.05 * dy;

    set_coords();
}


/**
 * Scrolls down the drawable area.
 */
void curvedlg::btnScrollDown_clicked()
{
    long double dy;

    dy = (c_y2 - c_y1);

    c_y2 -= 0.05 * dy;
    c_y1 -= 0.05 * dy;

    set_coords();
}


/**
 * Scrolls to left the drawable area.
 */
void curvedlg::btnScrollLeft_clicked()
{
    long double dx;

    dx = (c_x2 - c_x1);

    c_x2 -= 0.05 * dx;
    c_x1 -= 0.05 * dx;

    set_coords();
}


/**
 * Scrolls to right the drawable area.
 */
void curvedlg::btnScrollRight_clicked()
{
    long double dx;

    dx = (c_x2 - c_x1);

    c_x2 += 0.05 * dx;
    c_x1 += 0.05 * dx;

    set_coords();
}


/**
 * Zooms in the drawable area.
 */
void curvedlg::btnZoomPlus_clicked()
{
    long double dx,dy;

    dx = (c_x2 - c_x1);
    dy = (c_y2 - c_y1);

    c_x1 += 0.05 * dx;
    c_x2 -= 0.05 * dx;
    c_y1 += 0.05 * dx;
    c_y2 -= 0.05 * dy;

    set_coords();
}


/**
 * Zooms out the drawable area.
 */
void curvedlg::btnZoomMinus_clicked()
{
    long double dx,dy;

    dx = (c_x2 - c_x1);
    dy = (c_y2 - c_y1);

    c_x1 -= 0.05 * dx;
    c_x2 += 0.05 * dx;
    c_y1 -= 0.05 * dx;
    c_y2 += 0.05 * dy;

    set_coords();
}



/**
 * Slot is called when SavePNG button clicked.
 * Calls the image saving function.
 */
void curvedlg::btnSavePNG_clicked()
{
   drawWidget->save_as_PNG();
}
