/***************************************************************************
 *   Copyright (C) 2004 by Janos Toberling                                 *
 *   tobi54@users.sourceforge.net                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdlib.h>
#include "drawwidget.h"

/**
  * The width of the image buffer (in pixel)
  */
#define HSIZE 500

/**
  * The number of horizontal splits int the coordinate system
  */
#define HSPLIT 10

/**
  * The hight of the image buffer (in pixel)
  */
#define VSIZE 250

/**
  * The number of vertical splits int the coordinate system
  */
#define VSPLIT 10

/**
 * Drawwidget main contstructor. Initializes the drawing area.
 */
drawwidget::drawwidget(QWidget *parent, const char *name)
 : QWidget(parent, name)
{
    setBackgroundMode(PaletteBase);
    setPaletteBackgroundColor( QColor(0,0,0) );

    resize( HSIZE, VSIZE );
    buffer.resize( HSIZE, VSIZE );

    setMouseTracking(TRUE);
}


/**
 * Standard destructor.
 */
drawwidget::~drawwidget()
{
#ifndef QT_NO_PRINTER
    delete printer;
#endif
}

/**
 * Creates the connection between the drawwidget and the matdata object.
 * @param pmatdata pointer to matdata class.
 */
void drawwidget::initmatdata(matdata *pmatdata)
{
    matData = pmatdata;
}

/**
 * Draws the coordinate system.
 * @param p Painter device.
 */
void drawwidget::drawcoord(QPainter *p)
{
    int i,x,y;
    double long fx,fy;
    QString n;
    long double mul1,mul2;

    QPen lb (QColor(0,128,255),1);
    QPen g  (QColor(0,128,0),1);
    QPen lg  (QColor(0,255,0),1);


    QFont f( "times", 10, QFont::Bold );
    p->setFont( f );

    mul1 = (c_x2 - c_x1);
    for(i=0;i<HSPLIT;i++){
        fx = (long double)i * ((long double)HSIZE / (long double)HSPLIT );

        p->setPen( QPen( QColor(0,32,0) ,1 ) );
        p->drawLine((int)fx,0,(int)fx,VSIZE);

        p->setPen( QPen( g));
        p->drawLine((int)fx,0,(int)fx,6);
        mul2 = (long double)((long double)i / HSPLIT);
        n.sprintf( "%.3Lg", c_x1 + mul1 * mul2  );
        p->setPen(lb);
        p->drawText( (int)fx, 10, n );
    }

    mul1 = (c_y2 - c_y1);
    for(i=0;i<VSPLIT;i++){
        fy = (long double)VSIZE - ( (long double)i * ( (long double)VSIZE / (long double)VSPLIT ));

        p->setPen( QPen( QColor(0,32,0) ,1 ) );
        p->drawLine(0,(int)fy,HSIZE,(int)fy);

        p->setPen( QPen( g));
        p->drawLine(0,(int)fy,6,(int)fy);

        mul2 = (long double)((long double)i / (long double)VSPLIT);
        n.sprintf( "%.3Lg", c_y1 + mul1 * mul2  );
        p->setPen(lb);
        p->drawText( 0, (int)fy, n );
    }

    if ((c_x1 < 0) & (c_x2 > 0)) {
        x = (int) (       HSIZE *  ( 0 - c_x1) / (c_x2 - c_x1)    );
        p->setPen(lg);
        p->drawLine(x,0,x,VSIZE);

    }
    if ((c_x1 > 0) & (c_x2 < 0)) {
        x = (int) (       HSIZE *  ( 0 - c_x2) / (c_x1 - c_x2)    );
        p->setPen(lg);
        p->drawLine(x,0,x,VSIZE);

    }
    if ((c_y1 < 0) & (c_y2 > 0)) {
        y = (int) (       VSIZE *  ( 0 - c_y1) / (c_y2 - c_y1)    );
        p->setPen(lg);
         p->drawLine(0,VSIZE-y,HSIZE,VSIZE-y);

    }
    if ((c_y2 < 0) & (c_y1 > 0)) {
        y = (int) (       VSIZE *  ( 0 - c_y2) / (c_y1 - c_y2)    );
        p->setPen(lg);
        p->drawLine(0,VSIZE-y,HSIZE,VSIZE-y);

    }

}

/**
 * Draws the coordinate points
 * @param p Painter device
 */
void drawwidget::drawpoints(QPainter *p)
{
    int i,j;
    int x,y;

    QPen r  (QColor(255,64,255),1);
    p->setPen(r);

    for(i=0;i<matData->num_points;i++) {
        x = X_coord_to_pos(matData->points[i].x);
        y = Y_coord_to_pos(matData->points[i].y);
        for(j=1;j<5;j++) {
            p->drawPoint(x-j,y-j); p->drawPoint(x+j,y-j);
            p->drawPoint(x+j,y+j); p->drawPoint(x-j,y+j);
        }


    }
}


// Called when the print button is clicked.
//

/**
 * Currently unused.
 */
void drawwidget::printIt()
{
    if ( printer->setup( this ) ) {
        QPainter paint;
        if( !paint.begin( printer ) )
            return;
//        drawShapes( &paint );
    }
}


/**
 * Called when the widget needs to be updated.
 */
void drawwidget::paintEvent( QPaintEvent * )
{
    QPainter paint( this );
    QPainter p ( &buffer );

    buffer.fill( QColor(0,0,0) );
    drawcoord(&p);
    drawPolynome(&p);
    drawpoints(&p);

    bitBlt(this, 0,0 , &buffer, 0,0, HSIZE, VSIZE );
}


/**
 * Currently unused
 */
void drawwidget::resizeEvent( QResizeEvent * )
{
//    bgroup->move( width()-bgroup->width(), 0 );
}

/**
 * Function to set the coordinates of the drawing area
 * @param p_x1 bottom left X coordinate
 * @param p_y1 bottom left Y coordinate
 * @param p_x2 upper right X coordinate
 * @param p_y2 upper right Y coordinate
 */
void drawwidget::set_coords(long double p_x1, long double p_y1, long double p_x2, long double p_y2)
{
    c_x1 = p_x1;
    c_x2 = p_x2;
    c_y1 = p_y1;
    c_y2 = p_y2;

    this->update();
}

/**
    * Function to tranlate the mouse coordinate to drawing area.
    * @param  X coordinate of the mouse in the widget.
    * @return X coordinate of the mouse in the drawing coordinate system.
    */
long double drawwidget::X_pos_to_coord(int x)
{
    long double retX;

    retX  = (c_x2 - c_x1) * (long double)((long double)x / HSIZE);
    retX += c_x1;

    return (retX);
}

/**
    * Function to tranlate the mouse coordinate to drawing area.
    * @param  Y coordinate of the mouse in the widget.
    * @return Y coordinate of the mouse in the drawing coordinate system.
    */
long double drawwidget::Y_pos_to_coord(int y)
{
    long double retY;

    retY  = (c_y2 - c_y1) * (long double)((long double)(VSIZE-(long double)y) / VSIZE);
    retY += c_y1;

    return (retY);

}

/**
    * Function to tranlate the drawing area coordinate to a mouse coordinate.
    * @param   X coordinate of the mouse in the drawing coordinate system.
    * @return  X coordinate of the mouse in the widget.
    */
int drawwidget::X_coord_to_pos(long double px)
{
    long double rx;

    rx = (long double)(HSIZE * ( px - c_x1) / (c_x2 - c_x1));

    return ((int)rx);
}

/**
    * Function to tranlate the drawing area coordinate to a mouse coordinate.
    * @param   Y coordinate of the mouse in the drawing coordinate system.
    * @return  Y coordinate of the mouse in the widget.
    */
int drawwidget::Y_coord_to_pos(long double py)
{
    long double ry;

    ry = (long double) (VSIZE - (VSIZE * ( py - c_y1) / (c_y2 - c_y1)));

    return ((int)ry);
}


/**
 * Draws the line of the function
 * @param p Painter device
 */
void drawwidget::drawPolynome(QPainter *p)
{
    int x,yy,py;
    long double y,ty,f_x;
    int i;

    QPen yell(QColor(255,255,32),1);
    p->setPen(yell);

    for(x=0;x<HSIZE;x++) {

        y = 0;
        for(i=0;i<=matData->num_ad;i++){
            f_x = X_pos_to_coord(x);
            ty = matData->coefs[i] * powl(f_x,i);
            y += ty;
        }
        yy = Y_coord_to_pos(y);

        if (yy<0) yy=-1;
        if (yy>VSIZE) yy=VSIZE;

            if (( abs(py-yy)>1 ) && (x>1))
                p->drawLine(x-1,py,x,yy);
            else
                p->drawPoint(x,yy);

            py = yy;


    }

}

/**
 * Sends a qmousecoordsevenet when mouse is moving in the drawing area
 * @param e mouse event
 */
void drawwidget::mouseMoveEvent( QMouseEvent *e)
{
    qmousecoordsevent *mce = new qmousecoordsevent(e->pos());
    QApplication::postEvent( parent()->parent()->parent(), mce );

}

/**
 * Displays an add_coordinate menu if the mouse is pressed in the drawing area
 * @param e mouse event
 */
void drawwidget::mousePressEvent( QMouseEvent *e)
{
    QString s;

    QPopupMenu * addmenu = new QPopupMenu(this);

    addmenu->insertItem( i18n("Add point"), this, SLOT(add_point()) );


    mouse_x = X_pos_to_coord( e->x()) ;
    mouse_y = Y_pos_to_coord( e->y()) ;
    s.sprintf("( %13.4Lg ; %13.4Lg )",mouse_x, mouse_y);
    addmenu->insertItem( s, 0);
//    addmenu->setItemEnabled(0,FALSE);

//    addmenu->show();
    //addmenu->setFocus();

    addmenu->exec( QCursor::pos() );

    //addmenu->close();
}


/**
    * Save the drawing area to an image file.
    */
void drawwidget::save_as_PNG()
{
    QString filename = QFileDialog::getSaveFileName( QString::null, QString( "*.png *.PNG"), this );

  if ( !filename.isEmpty() ) {

            if ( QString::compare( (filename.lower().right( 4 ) ), ".png" )) {
                filename += ".png";
            }

            int answer = 0;
            if ( QFile::exists( filename ) )
                answer = QMessageBox::warning(
                                this,
                                tr2i18n("PNG file %1 exisits.").arg( filename ),
                                tr2i18n( "Overwrite\n\'%1\'?" ).arg( filename ),
                                tr2i18n("&Yes"), tr2i18n("&No"), QString::null, 1, 1 );
            if ( answer == 0 ) {
                // Save
                buffer.save( filename, "PNG", -1);
                return;
            }
    }
}

/**
    * Add a coordinate pair - based on the mouse coordinates - to the datavector.
    */
void drawwidget::add_point()
{
    int num;

    num = ++(matData->num_points);

    matData->points[num-1].x = mouse_x;
    matData->points[num-1].y = mouse_y;

    matData->calculate();

    addpointevent *ape = new addpointevent(mouse_x,mouse_y);
    QApplication::postEvent( parent()->parent()->parent()->parent(), ape );

}

#include "drawwidget.moc"
