/***************************************************************************
 *   Copyright (C) 2004 by Janos Toberling                                 *
 *   tobi54@users.sourceforge.net                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DRAWWIDGET_H
#define DRAWWIDGET_H

#include <qdialog.h>
#include <qpainter.h>
#include <qprinter.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qapplication.h>
#include <math.h>
#include <qwidget.h>
#include <qpixmap.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <klocale.h>
#include <qpopupmenu.h>
#include <qcursor.h>

#include "qmousecoordsevent.h"
#include "matdata.h"

/**
* This class provides the drawing of the function.
@author Janos Toberling
*/
class drawwidget : public QWidget
{
Q_OBJECT
public:
    drawwidget(QWidget *parent = 0, const char *name = 0);

    ~drawwidget();

public slots:
    /**
     * *** Currently unused ***
     */
    void            printIt();

    /**
     * Function to set the coordinates of the drawing area
     * @param  bottom left X coordinate
     * @param  bottom left Y coordinate
     * @param  upper right X coordinate
     * @param  upper right Y coordinate
     */
    void            set_coords(long double, long double, long double, long double);

    /**
     * Function to tranlate the mouse coordinate to drawing area.
     * @param  X coordinate of the mouse in the widget.
     * @return X coordinate of the mouse in the drawing coordinate system.
     */
    long double     X_pos_to_coord(int);

    /**
     * Function to tranlate the mouse coordinate to drawing area.
     * @param  Y coordinate of the mouse in the widget.
     * @return Y coordinate of the mouse in the drawing coordinate system.
     */
    long double     Y_pos_to_coord(int);

    /**
     * Function to tranlate the drawing area coordinate to a mouse coordinate.
     * @param   X coordinate of the mouse in the drawing coordinate system.
     * @return  X coordinate of the mouse in the widget.
     */
    int             X_coord_to_pos(long double);

    /**
     * Function to tranlate the drawing area coordinate to a mouse coordinate.
     * @param   Y coordinate of the mouse in the drawing coordinate system.
     * @return  Y coordinate of the mouse in the widget.
     */
    int             Y_coord_to_pos(long double);

    /**
    * Creates the connection between the drawwidget and the matdata object.
    * @param pmatdata pointer to matdata class.
    */
    void            initmatdata(matdata *);

    /**
     * Save the drawing area to an image file.
     */
    void            save_as_PNG();

    /**
     * Add a coordinate pair - based on the mouse coordinates - to the datavector.
     */
    void            add_point();

protected:
    /**
    * Called when the widget needs to be updated.
    */
    void            paintEvent( QPaintEvent * );

    /**
    * Currently unused.
    */
    void            resizeEvent( QResizeEvent * );

    /**
    * Draws the coordinate system.
    * @param p Painter device.
    */
    void            drawcoord( QPainter *);

    /**
    * Draws the datacoordinate points
    * @param p Painter device
    */
    void            drawpoints ( QPainter *);

    /**
    * Draws the line of the function
    * @param p Painter device
    */
    void            drawPolynome (QPainter *);

    /**
    * Sends a qmousecoordsevenet when mouse is moving in the drawing area
    * @param e mouse event
    */
    void            mouseMoveEvent( QMouseEvent *);

    /**
    * Displays an add_coordinate menu if the mouse is pressed in the drawing area
    * @param e mouse event
    */
    void            mousePressEvent( QMouseEvent *);


private:
    /**
      * Currently unused printer device
      */
    QPrinter        *printer;

    QButtonGroup    *bgroup;
    QPushButton     *print;

    /**
      * Drawing area coordinates
      */
    long double     c_x1,c_x2,c_y1,c_y2;
    /**
      * Pointer to mathematical data
      */
    matdata         *matData;
    /**
      * The drawing buffer
      */
    QPixmap         buffer;

    /**
      * Holder for mouse coordinate
      */
    long double     mouse_x, mouse_y;

};

#endif
