/*******************************************************************
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/

/**
 * Special validator for coordinate input X
 * @param inpX The changed text value.
 */
void maindlg::inputX_textChanged( const QString & inpX)
{
    bool ok;

    dX = inpX.toDouble( &ok );

    if (ok) {
        okX = TRUE;
        btnAdd->setEnabled(TRUE);
        inputX->setPaletteForegroundColor( QColor(0,0,0) );
    } else
    {
        okX = FALSE;
        btnAdd->setEnabled(FALSE);
        inputX->setPaletteForegroundColor( QColor(255,0,0) );
    }

    if (okX && okY)
        btnAdd->setEnabled(TRUE);
    else
        btnAdd->setEnabled(FALSE);
}


/**
 * Special validator for coordinate input Y
 * @param inpX The changed text value.
 */
void maindlg::inputY_textChanged( const QString & inpY)
{
    bool ok;

    dY = inpY.toDouble( &ok );

    if (ok) {
        okY = TRUE;
        inputY->setPaletteForegroundColor( QColor(0,0,0) );
    } else
    {
        okY = FALSE;
        inputY->setPaletteForegroundColor( QColor(255,0,0) );
    }

    if (okX && okY)
        btnAdd->setEnabled(TRUE);
    else
        btnAdd->setEnabled(FALSE);

}


/**
 * Slot for add a data coordinate pair
 */
void maindlg::add_coord()
{
    char chbuff[100];

    QListViewItem * item = new QListViewItem(coordlist, 0);

    sprintf(chbuff, "%13.4g",dX);
    item->setText(0, QString( chbuff ));

    sprintf(chbuff, "%13.4g",dY );
    item->setText(1, QString( chbuff ));

    coordlist->setSelected(item,TRUE);

    btnModify->setEnabled(TRUE);
    btnDelete->setEnabled(TRUE);

    /* FIX some buggy condition */
   if (coordlist->childCount() == 1) {
        labelMaxNr->setText( "0" ) ;
        boxNr->setMaxValue(0);
        boxNr->setEnabled(TRUE);
    }

    coordlist_changed();

}


/**
 * Select a coordinate to modify
 */
void maindlg::coordlist_clicked( QListViewItem * )
{
     QListViewItem *item = coordlist->currentItem();

     if (item) {
        inputX->setText( item->text(0) .stripWhiteSpace());
        inputY->setText( item->text(1) .stripWhiteSpace());

        btnModify->setEnabled(TRUE);
        btnDelete->setEnabled(TRUE);
     }
}


/**
 * Initialization. Creates curvedialog object.
 */
void maindlg::init()
{
    dX = 0;
    dY = 0;

    okX = FALSE;
    okY = FALSE;

    matData = 0;
    curveDialog = new curvedialog(this);
}


/**
 * Slot to delete a selected coordinate.
 */
void maindlg::btnDelete_clicked()
{
    QListViewItem * item = coordlist->selectedItem();
    if ( item ) {
        item->~QListViewItem();

        btnModify->setEnabled(FALSE);
        btnDelete->setEnabled(FALSE);
    }
    coordlist_changed();
}


/**
 * Slot to change a selected coordinate
 */
void maindlg::btnModify_clicked()
{
    char chbuff[100];

    QListViewItem * item = coordlist->selectedItem();
    if ( item ) {
        sprintf(chbuff, "%13.4g",dX);
        item->setText(0, QString( chbuff ));

        sprintf(chbuff, "%13.4g",dY );
        item->setText(1, QString( chbuff ));
    }

    coordlist_changed();
}


/**
 * If datapairs changed we should recaulculate the results.
 */
void maindlg::coordlist_changed()
{
    char chrs[100];
    QString s;
    int datasets,cntr;
    QListViewItem   *item;
    QString d1, d2;
    long double dx,dy;

    s = "--";
    datasets = 0; cntr = 0;

    item = coordlist->firstChild();
    while (item) {

        dx = item->text(0).toDouble();
        dy = item->text(1).toDouble();
        matData->points[cntr].x = dx;
        matData->points[cntr].y = dy;

        cntr ++;

        d1 = s.ascii();
        d2 = item->text(0).ascii();
        if ( strcmp(d1,d2) )
                datasets++;
        s = item->text(0);

        item = item->nextSibling();

    }

    sprintf(chrs,"%i",cntr);
    s = chrs;
    labelTotalNr->setText( s );
    matData->num_points = cntr;

    if (--datasets>=0) {
        sprintf(chrs, "%i", datasets );
        s =  chrs;

        boxNr->setMaxValue(datasets);
        boxNr->setEnabled(TRUE);
    } else {
        s = " -- " ;
        boxNr->setEnabled(FALSE);
    }

    labelMaxNr->setText( s );

    matData->calculate();
    set_coefs();
}


/**
 * If the searched power changed we have to recaulculate the results.
 */
void maindlg::index_changed()
{
    bool ok;
    int num_index;

    num_index = boxNr->text().toInt(&ok,10);
    matData->num_ad = num_index;

    matData->calculate();
    set_coefs();

}


/**
 * Notify curvedialog about the initialization of mathematical structures.
 * @param pmatdata
 */
void maindlg::init_matdata( matdata *pmatdata )
{
    matData = pmatdata;
    curveDialog->initmatdata(pmatdata);
}


/**
 * Display coefficients
 */
void maindlg::set_coefs()
{
    QListViewItem *item, *next;
    int i;
    char chbuff[100];

    item  = taglist->firstChild();
    while (item) {
        next = item->nextSibling();
        item->~QListViewItem();
        item = next;
    }

    for(i=0; i <=matData->num_ad;i++) {

        item = new QListViewItem(taglist, 0);

        sprintf(chbuff,"x^%i",i);
        item->setText(0, QString( chbuff ));

        sprintf(chbuff, "%13.4Lg",matData->coefs[i] );
        item->setText(1, QString( chbuff ));
    }

    sprintf(chbuff,"%13.4Lg",matData->sigma);
    le_sigma->setText( QString( chbuff ) );

    sprintf(chbuff,"%13.4Lg",matData->sigma_star);
    le_sigma_star->setText( QString( chbuff ) );

    curveDialog->matdata_changed();
}


/**
 * Inititate the drawing of the fuction curve.
 */
void maindlg::showgraph()
{
  curveDialog->show();
}


/**
 * Save the parameters.
 */
void maindlg::btnSave_clicked()
{
    bool    SaveOK;
    FILE    *fout;
    int     i;

    SaveOK = FALSE;

    QString s = QFileDialog::getSaveFileName(
        ".",
        tr2i18n("KPolygon_datasets (*.kpds)"),
        this,
        tr2i18n("Choose a filename to save dataset")        );

        if ( QString::compare(s.right( 5 ), ".kpds" )) {
            s += ".kpds";
        }

    if (!s.isEmpty()) {

        fout = fopen( s.ascii(), "rw" );
        if (fout) {
            fclose(fout);
            if ( QMessageBox::question( this, tr2i18n("Overwrite file?"),
                tr2i18n("Filename %1 already exists."
                " Do you want to overwrite?").arg(s.ascii()), tr2i18n("&Yes"), tr2i18n("&No"), QString::null))
                return;


        }

        fout = fopen( s.ascii(), "w" );
        if (fout) {
            fprintf(fout,"Num_points: %i\n",matData->num_points);
            for(i=0;i<matData->num_points;i++) {
                fprintf(fout,"( %13.4Lg ; %13.4Lg )\n",matData->points[i].x, matData->points[i].y );
            }
            fprintf(fout,"Num_ads: %i\n",matData->num_ad);
            fprintf(fout,"------ Calculated_data ------\n");
            for(i=0;i<=matData->num_ad;i++) {
                fprintf(fout,"x^%i = %13.4Lg\n",i,matData->coefs[i]);
            }
    fprintf(fout,"\nsigma = %13.4Lg          sigma^* = %134Lg\n", matData->sigma, matData->sigma_star);

            fclose(fout);
            SaveOK = TRUE;
        }
        if (SaveOK)
            QMessageBox::information(this,"KPolynome", tr2i18n("Datasets saved successfully."));
        else
            QMessageBox::warning(this,"KPolynome",tr2i18n("Datasets not saved!") );

    }

}


/**
 * Load the parameteres
 */
void maindlg::btnLoad_clicked()
{
    QListViewItem *item, *next;
    bool    LoadOK;
    FILE    *fin;
    int     i;
    char    chbuff[100];

    LoadOK = FALSE;

    QString s = QFileDialog::getOpenFileName(
        ".",
        tr2i18n("KPolygon_datasets (*.kpds)"),
        this,
        tr2i18n("Choose a filename to load dataset")        );

        if (!s.isEmpty()) {
            fin = fopen( s.ascii(), "r" );
            if (fin) {

                item  = coordlist->firstChild();
                while (item) {
                    next = item->nextSibling();
                    item->~QListViewItem();
                    item = next;
                }


                fscanf(fin,"Num_points: %i\n",&matData->num_points);
                for(i=0;i<matData->num_points;i++) {
                    fscanf(fin,"( %Lg ; %Lg )\n",&matData->points[i].x, &matData->points[i].y );
                    item = new QListViewItem(coordlist,0);
                    sprintf(chbuff,"%13.4Lg",matData->points[i].x);
                    item->setText( 0, QString( chbuff ) );
                    sprintf(chbuff,"%13.4Lg",matData->points[i].y);
                    item->setText( 1, QString( chbuff ) );
                }

                fscanf(fin,"Num_ads: %i\n",&matData->num_ad);
                boxNr->setValue( matData->num_ad );

                fclose(fin);

                LoadOK = TRUE;

                coordlist_changed();
                index_changed();
            }
            if (LoadOK)
                QMessageBox::information(this,"KPolynome", tr2i18n("Dataset load successfully."));
            else
                QMessageBox::warning(this,"KPolynome",tr2i18n("Datasets not loaded!") );
        }

}



/**
 * Event filter to add a coordinate pair from the drawable area.
 * @param e Event
 */
bool maindlg::event( QEvent *e )
{
    QString s;

    if ( e->type() == ADDPOINTEVENT ) {
            addpointevent * ape = (addpointevent *)e;

            dX = (double) ape->x();
            okX = TRUE;
            s.sprintf("%13.4Lg",(long double)dX);
            inputX->setText(s);

            dY = (double) ape->y();
            okY = TRUE;
            s.sprintf("%13.4Lg",(long double)dY);
            inputY->setText(s);

            add_coord();

            return TRUE;
    } else
            return QDialog::event(e);

}
