/***************************************************************************
 *   Copyright (C) 2004 by Janos Toberling                                 *
 *   tobi54@users.sourceforge.net                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "matdata.h"

#include <stdio.h>
#include <math.h>

/**
 * The constructor of the matdata class. Nulls some internal variables.
 * @return none.
 */
matdata::matdata()
{
  int i;

  for(i=0;i<MAXPOINTS;i++)  {
    points[i].x = 0;
    points[i].y = 0;
    coefs[i]    = 0;
  }

  ppoints = &points;

  num_ad    = 0;
  num_points    = 0;

}

/**
 * The default destructor
 * @return
 */
matdata::~matdata()
{}

/**
 * Dumps out the data point to stdout.
 * @return
 */
void matdata::dump_points()
{
    int i;

    printf("\n----- Dump of points ---------\n");
    printf("Keresett fok: %i\n",num_ad);

    for(i=0;i<num_points;i++) {
                printf("%i.   ( %7.2Lf , %7.2Lf )\n", i, points[i].x, points[i].y) ;
        }

}


/**
 * Calculates the results.
 * Fills the calculation matrix, creates a diagonal and calculates the divergences.
 * @return none.
 */
void matdata::calculate()
{
    int i;
    long double y_diff;

    fill_matrix();
    diag_matrix();

    for(i=0;i<=num_ad;i++) {
      printf("coef[%i] := %Lf\n",num_ad - i,cmatrix[num_ad+1][i] );
        coefs[num_ad - i] = cmatrix[num_ad+1][i];
    }

    /* calculate ro */

    y_diff = 0;
    for(i=0;i<num_points;i++) {
        y_diff +=  fabsl( points[i].y - value( points[i].x ) ) ;
    }

    sigma      = num_points>0?y_diff / num_points:0;
    sigma_star = num_points>1?y_diff / (num_points -1 ):0;

}

/**
 * Internal function to dump the calculation matrix to stdout.
 */
void matdata::dump_matrix()
{
        int x,y;

        printf("\n----- 2.Dump of the calculation matrix ---------");

        printf("\n ");
        for(y=0;y<=num_ad;y++) {

                for(x=0;x<=num_ad;x++) {
                        printf("%Lf ", cmatrix[x][y] );
                }
                printf(" | %Lf\n ", cmatrix[x][y] );
        }

        printf("\n");

}

/**
 * Fills the left and right side of the matrix.
 *
 */
void matdata::fill_matrix()
{
        fill_left();
        fill_right();
}

/**
 * Creates the diagonal.
 */
void matdata::diag_matrix()
{
        int     d_n;

        for(d_n=0;d_n<num_ad+1;d_n++) {

                create1line(d_n);
                sub_lines(d_n);

        }
  // dump_matrix();
}


/**
 * Calculates the result of polinome in the point of the parameter.
 * @return long double result_of_the_polinome
 */
long double matdata::value(long double px)
{
        int i;
        long double retf;

        retf = 0;

        for(i=0;i<=num_ad;i++) {
            retf += coefs[i] * powl(px,i);
        }
        return (retf);
}

/**
 * Function to help to calculate a matrix element durint the fill.
 * @param ad_n searched power
 * @return the sum of all X coordinates in the given power
 */
long double matdata::xsum_pow_n(int ad_n)
{
        int i;
        long double retf;

        retf = 0;

        for(i=0;i<num_points;i++) {
                retf += pow(points[i].x, ad_n);
        }

        return (retf);
}


/**
 * Function to help to calculate a matrix element durint the fill.
 * @param ad_n searched power
 * @return the sum of all X coordinates in the given power multiplied by the Y coordinates
 */
long double matdata::yxsum_pow_n(int ad_n)
{
        int i;
        long double retf;

        retf = 0;

        for(i=0;i<num_points;i++) {
                retf += points[i].y * pow(points[i].x, ad_n);
        }

        return (retf);
}

/**
 * Calculates the left side of the matrix.
 */
void matdata::fill_left()
{
        int x,y;

        for(x=0;x<=num_ad;x++)
         for(y=0;y<=num_ad;y++) {
         cmatrix[x][y] = xsum_pow_n(2*num_ad -x -y );
        }

}

/**
 * Calculates the right side of the matrix.
 */
void matdata::fill_right()
{
        int y;

        for(y=0;y<=num_ad;y++) {
                cmatrix[num_ad+1][y] = yxsum_pow_n(num_ad - y) ;
        }
}

/**
 * Generates a row that can be subsctraced from the others.
 * @param d_n Describes the (d_n, d_n) position of the generator element.
 */
void matdata::create1line(int d_n)
{
    int     x;
    long double multier;

    multier = cmatrix[d_n][d_n];

    for(x=0;x<=num_ad+1;x++)
          cmatrix[x][d_n] = cmatrix[x][d_n] / multier ;     // Let's given row divide by the tag ([dn];[dn])
}

/**
 * Subsctract a given line form the matrix to null elements.
 * @param d_n the number of the line to substract from the others.
 */
void matdata::sub_lines(int d_n)
{
        int x,y;
        long double multier;

        for(y=0;y<num_ad+1;y++)                         // For all columns
                if  ( y != d_n)                         // If it's not mee!
                {
                        multier = cmatrix[d_n][y];
                        for(x=d_n;x<=num_ad+1;x++)      // With the entire row
                                cmatrix[x][y] -= multier * cmatrix[x][d_n];
                }
}
