/***************************************************************************
 *   Copyright (C) 2004 by Janos Toberling                                 *
 *   tobi54@users.sourceforge.net                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MATDATA_H
#define MATDATA_H

/**
 * The MAXPOINTS define is an upper limit for the number of the datapoint.
 */
#define MAXPOINTS 1024

/**
 * This structure holds a datacoordinate pair.
 */
typedef struct typ_point  {
         long double    x;          // X coordinate
         long double    y;          // Y coordinate
} ;


/**
 * A type to hold all the data points.
 */
typedef typ_point       typ_points[MAXPOINTS];  // Vector for datapoints
typedef typ_points *    typ_ptr_points;     // Pointer to the vector

/**
 * A 2D matrix type for the calculations
 */
typedef long double     typ_cmatrix[MAXPOINTS+1][MAXPOINTS];



/**
This class stores the varianles and the funcitons needed to generate the polinome.
@author Janos Toberling
*/
class matdata
{
public:
    matdata();

    ~matdata();

    /**
     * A vector to hold the coordinate points.
     */
    typ_point       points[MAXPOINTS];


    /**
     * A pointer to the coordinate-pair vector
     */
    typ_ptr_points  ppoints;

    /**
     * The number of the datapairs
     */
    int             num_points;             // Number of coordinate pairs
    /**
     * The searched degree
     */
    int             num_ad;                 // Maximum power
    /**
     * A 2D matrix for the calculations.
     */
    typ_cmatrix     cmatrix;            // The calculation matrix
    /**
     * The vector to hold the calculated coefficients.
     */
    long double     coefs[MAXPOINTS];        // Vector for calculated coefficients

    /**
     * The empiric divergence
     */
    long double     sigma;
     /**
     * The corrected empiric divergence
     */
    long double     sigma_star;

    void            calculate()   ;
    void            dump_points() ;
    void            dump_matrix() ;
    void            fill_matrix() ;
    void            diag_matrix() ;

    long double     value( long double px);

private:
    long double     xsum_pow_n(int ad_n);
    long double     yxsum_pow_n(int ad_n);

    void            fill_left();
    void            fill_right();
    void            create1line(int d_n);
    void            sub_lines(int d_n);

};

#endif
