/***************************************************************************
 *   Copyright (C) 2004 by Janos Toberling                                 *
 *   tobi54@users.sourceforge.net                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef QMOUSECOORDSEVENT_H
#define QMOUSECOORDSEVENT_H

#include <qevent.h>
#include <qpoint.h>


/**
* This event is generated in drawwidget class when the mouse moves in the drawing area
* @author Janos Toberling
*/

#define QMOUSECOORDEVENT 65432

class qmousecoordsevent : public QCustomEvent
{
public:
    qmousecoordsevent(QPoint pos)
        : QCustomEvent( QMOUSECOORDEVENT ), p( pos ) {}

    /**
     * @return QPoint: Mouse poisition
     */
    QPoint pos() const { return p; }

private:
    /**
      * Mouse position
      */
    QPoint p;

};

#define ADDPOINTEVENT 65431

/**
* This event is generated in drawwidget class when the mouse button is toggled
@author Janos Toberling
*/
class addpointevent : public QCustomEvent
{
public:
    addpointevent(long double px, long double py)
        : QCustomEvent( ADDPOINTEVENT ), xx(px), yy(py) {}

    /**
      * @return Mouse X position
      */
    long double x() const { return xx; }

    /**
      * @return Mouse Y position
      */
    long double y() const { return yy; }

private:
    /**
      * Mouse X position
      */
    long double xx;

    /**
      * Mouse Y position
      */
    long double yy;

};



#endif

